#!/usr/bin/env python
# coding: utf-8

# In[ ]:


#Import 

import pandas as pd
import matplotlib.pyplot as plt 
import numpy as np
import scipy as sp
import IPython
import sklearn

from platform import python_version

print('Python version', python_version())
print('The scikit-learn version:{}.'.format(sklearn.__version__))


# In[ ]:


#Import Dataset

jejuni = pd.read_csv ('Supplementary_file_3_Peak_matching_table_ML.csv')
print(jejuni.groupby('ST').size())

Y = jejuni['ST']
X = jejuni.drop(columns=['ST','ID'])


# In[ ]:


#Features transformation/Scaling

from sklearn.preprocessing import MinMaxScaler

scaler = MinMaxScaler()
jejuni_scaled = scaler.fit_transform(X)
print('Scaled dataframe',jejuni_scaled)


# In[ ]:


#Save the new dataframe

jejuni_scaled_df= pd.DataFrame(jejuni_scaled, Y)
jejuni_scaled_df
pd.DataFrame(jejuni_scaled, Y).to_csv('jejuni_scaled_df.csv')

#New variables

jejuni_2 = pd.read_csv ('jejuni_scaled_df.csv')
print(jejuni.groupby('ST').size())
Y = jejuni_2['ST']
X = jejuni_2.drop(columns=['ST'])


# In[ ]:


#Training/Validation/Test set

from sklearn.model_selection import train_test_split

X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size=0.3,stratify=Y,random_state=0, shuffle=True)

print('X_test:', X_test)
print ('X_test shape', X_test.shape)


# In[ ]:


#Evaluation/Train models

from sklearn import model_selection 
from sklearn.model_selection import cross_val_score
from sklearn.model_selection import StratifiedKFold
from sklearn.ensemble import RandomForestClassifier

plt.style.use('ggplot')
seed=0

models = []
models.append(('RF', RandomForestClassifier(random_state=seed)))

results = []
names = []
scoring = 'balanced_accuracy'
for name, model in models:
         kfold = model_selection.KFold(n_splits=10)
         cv_results = model_selection.cross_val_score(model, X_train, Y_train, cv=kfold, scoring=scoring)
         results.append(cv_results)
         names.append(name)
         msg = "%s: %f (%f)" % (name, cv_results.mean(), cv_results.std())
         print(msg)

fig = plt.figure()
fig.suptitle('Algorithm Comparison (10-fold cross-validation)')
ax = fig.add_subplot(111)
plt.boxplot(results)
ax.set_xticklabels(names)
plt.xlabel('Models')
plt.ylabel('Balanced accuracy')
plt.show() 


# In[ ]:


#Optimization of the Model

from sklearn.metrics import classification_report, confusion_matrix, ConfusionMatrixDisplay
from sklearn.metrics import recall_score
from sklearn.metrics import precision_score
from sklearn.metrics import f1_score
from sklearn.metrics import balanced_accuracy_score

RF = RandomForestClassifier(random_state=seed).fit(X_train,Y_train)

y_pred_RF = RF.predict(X_test)

confusion = confusion_matrix(Y_test,y_pred_RF)
print("Confusion matrix (RF):\n{}".format(confusion))
disp = ConfusionMatrixDisplay(confusion_matrix=confusion, display_labels=RF.classes_).plot()
plt.tick_params(axis=u'both', which=u'both',length=0)
plt.grid(b=None)

precision= precision_score(Y_test, y_pred_RF, average='macro')
recall=recall_score(Y_test, y_pred_RF, average='macro')
F1= f1_score(Y_test, y_pred_RF, average='macro')
BAcc=balanced_accuracy_score(Y_test, y_pred_RF)
print('F1-score:',F1)
print('Precision:',precision)
print('Recall:',recall)
print('B Accuracy:',BAcc)


# In[ ]:


#Features importance & Decision Tree

from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import GridSearchCV


jejuni = pd.read_csv ('Supplementary_file_3_Peak_matching_table_ML.csv')
print(jejuni.groupby('ST').size())

Y = jejuni['ST']
X = jejuni.drop(columns=['ST','ID'])

X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size=0.3,stratify=Y,random_state=0, shuffle=True)

#Tune Decision Tree

params = {'max_leaf_nodes': list(range(2, 100)),
          'min_samples_split': [2, 3, 4]}
grid_search_cv = GridSearchCV(DecisionTreeClassifier(), params, verbose=1, cv=10, scoring='balanced_accuracy')
grid_search_cv.fit(X_train, Y_train)

print(grid_search_cv.best_params_)
print(grid_search_cv.best_estimator_)


# In[ ]:


from matplotlib.pyplot import figure
from sklearn import tree

clf=DecisionTreeClassifier(ccp_alpha=0.0, class_weight=None, criterion='gini',
                       max_depth=None, max_features=None, max_leaf_nodes=45,
                       min_impurity_decrease=0.0, min_impurity_split=None,
                       min_samples_leaf=1, min_samples_split=4,
                       min_weight_fraction_leaf=0.0, presort='deprecated',
                       random_state=None, splitter='best').fit(X, Y)

columns= ['ST19','ST464','ST2254','ST6175']
datanames= list(X)
fig, axes = plt.subplots(figsize = (8, 9), dpi=600, sharex = True,sharey=True)
out = tree.plot_tree(clf, filled=True, feature_names=datanames, class_names=columns, impurity=True, rounded=True, proportion=False,fontsize=None)
for o in out:
    arrow = o.arrow_patch
    if arrow is not None:
        arrow.set_edgecolor('black')
        arrow.set_linewidth(0.8)


# In[ ]:





# In[ ]:





# In[ ]:




